@@ -9,12 +9,12 @@ use std::time::Duration;
 use curl_sys;
 use libc::c_void;
 
-use Error;
-use easy::{Form, List};
-use easy::handler::{self, InfoType, SeekResult, ReadError, WriteError};
-use easy::handler::{TimeCondition, IpResolve, HttpVersion, SslVersion};
-use easy::handler::{SslOpt, NetRc, Auth, ProxyType};
+use easy::handler::{self, InfoType, ReadError, SeekResult, WriteError};
+use easy::handler::{Auth, NetRc, ProxyType, SslOpt};
+use easy::handler::{HttpVersion, IpResolve, SslVersion, TimeCondition};
 use easy::{Easy2, Handler};
+use easy::{Form, List};
+use Error;
 
 /// Raw bindings to a libcurl "easy session".
 ///
@@ -105,13 +105,13 @@ unsafe impl Send for EasyData {}
 
 #[derive(Default)]
 struct Callbacks<'a> {
-    write: Option<Box<FnMut(&[u8]) -> Result<usize, WriteError> + 'a>>,
-    read: Option<Box<FnMut(&mut [u8]) -> Result<usize, ReadError> + 'a>>,
-    seek: Option<Box<FnMut(SeekFrom) -> SeekResult + 'a>>,
-    debug: Option<Box<FnMut(InfoType, &[u8]) + 'a>>,
-    header: Option<Box<FnMut(&[u8]) -> bool + 'a>>,
-    progress: Option<Box<FnMut(f64, f64, f64, f64) -> bool + 'a>>,
-    ssl_ctx: Option<Box<FnMut(*mut c_void) -> Result<(), Error> + 'a>>,
+    write: Option<Box<dyn FnMut(&[u8]) -> Result<usize, WriteError> + 'a>>,
+    read: Option<Box<dyn FnMut(&mut [u8]) -> Result<usize, ReadError> + 'a>>,
+    seek: Option<Box<dyn FnMut(SeekFrom) -> SeekResult + 'a>>,
+    debug: Option<Box<dyn FnMut(InfoType, &[u8]) + 'a>>,
+    header: Option<Box<dyn FnMut(&[u8]) -> bool + 'a>>,
+    progress: Option<Box<dyn FnMut(f64, f64, f64, f64) -> bool + 'a>>,
+    ssl_ctx: Option<Box<dyn FnMut(*mut c_void) -> Result<(), Error> + 'a>>,
 }
 
 impl Easy {
@@ -165,6 +165,11 @@ impl Easy {
         self.inner.unix_socket(unix_domain_socket)
     }
 
+    /// Same as [`Easy2::unix_socket_path`](struct.Easy2.html#method.unix_socket_path)
+    pub fn unix_socket_path<P: AsRef<Path>>(&mut self, path: Option<P>) -> Result<(), Error> {
+        self.inner.unix_socket_path(path)
+    }
+
     // =========================================================================
     // Callback options
 
@@ -232,7 +237,8 @@ impl Easy {
     /// transfer.perform().unwrap();
     /// ```
     pub fn write_function<F>(&mut self, f: F) -> Result<(), Error>
-        where F: FnMut(&[u8]) -> Result<usize, WriteError> + Send + 'static
+    where
+        F: FnMut(&[u8]) -> Result<usize, WriteError> + Send + 'static,
     {
         self.inner.get_mut().owned.write = Some(Box::new(f));
         Ok(())
@@ -303,7 +309,8 @@ impl Easy {
     /// transfer.perform().unwrap();
     /// ```
     pub fn read_function<F>(&mut self, f: F) -> Result<(), Error>
-        where F: FnMut(&mut [u8]) -> Result<usize, ReadError> + Send + 'static
+    where
+        F: FnMut(&mut [u8]) -> Result<usize, ReadError> + Send + 'static,
     {
         self.inner.get_mut().owned.read = Some(Box::new(f));
         Ok(())
@@ -334,7 +341,8 @@ impl Easy {
     /// `transfer` method and then using `seek_function` to configure a
     /// callback that can reference stack-local data.
     pub fn seek_function<F>(&mut self, f: F) -> Result<(), Error>
-        where F: FnMut(SeekFrom) -> SeekResult + Send + 'static
+    where
+        F: FnMut(SeekFrom) -> SeekResult + Send + 'static,
     {
         self.inner.get_mut().owned.seek = Some(Box::new(f));
         Ok(())
@@ -377,7 +385,8 @@ impl Easy {
     /// `transfer` method and then using `progress_function` to configure a
     /// callback that can reference stack-local data.
     pub fn progress_function<F>(&mut self, f: F) -> Result<(), Error>
-        where F: FnMut(f64, f64, f64, f64) -> bool + Send + 'static
+    where
+        F: FnMut(f64, f64, f64, f64) -> bool + Send + 'static,
     {
         self.inner.get_mut().owned.progress = Some(Box::new(f));
         Ok(())
@@ -413,7 +422,8 @@ impl Easy {
     /// `transfer` method and then using `progress_function` to configure a
     /// callback that can reference stack-local data.
     pub fn ssl_ctx_function<F>(&mut self, f: F) -> Result<(), Error>
-        where F: FnMut(*mut c_void) -> Result<(), Error> + Send + 'static
+    where
+        F: FnMut(*mut c_void) -> Result<(), Error> + Send + 'static,
     {
         self.inner.get_mut().owned.ssl_ctx = Some(Box::new(f));
         Ok(())
@@ -433,7 +443,8 @@ impl Easy {
     /// `transfer` method and then using `debug_function` to configure a
     /// callback that can reference stack-local data.
     pub fn debug_function<F>(&mut self, f: F) -> Result<(), Error>
-        where F: FnMut(InfoType, &[u8]) + Send + 'static
+    where
+        F: FnMut(InfoType, &[u8]) + Send + 'static,
     {
         self.inner.get_mut().owned.debug = Some(Box::new(f));
         Ok(())
@@ -517,7 +528,8 @@ impl Easy {
     /// println!("{:?}", headers);
     /// ```
     pub fn header_function<F>(&mut self, f: F) -> Result<(), Error>
-        where F: FnMut(&[u8]) -> bool + Send + 'static
+    where
+        F: FnMut(&[u8]) -> bool + Send + 'static,
     {
         self.inner.get_mut().owned.header = Some(Box::new(f));
         Ok(())
@@ -546,6 +558,11 @@ impl Easy {
         self.inner.port(port)
     }
 
+    /// Same as [`Easy2::connect_to`](struct.Easy2.html#method.connect_to)
+    pub fn connect_to(&mut self, list: List) -> Result<(), Error> {
+        self.inner.connect_to(list)
+    }
+
     /// Same as [`Easy2::proxy`](struct.Easy2.html#method.proxy)
     pub fn proxy(&mut self, url: &str) -> Result<(), Error> {
         self.inner.proxy(url)
@@ -556,6 +573,26 @@ impl Easy {
         self.inner.proxy_port(port)
     }
 
+    /// Same as [`Easy2::proxy_cainfo`](struct.Easy2.html#method.proxy_cainfo)
+    pub fn proxy_cainfo(&mut self, cainfo: &str) -> Result<(), Error> {
+        self.inner.proxy_cainfo(cainfo)
+    }
+
+    /// Same as [`Easy2::proxy_capath`](struct.Easy2.html#method.proxy_capath)
+    pub fn proxy_capath<P: AsRef<Path>>(&mut self, path: P) -> Result<(), Error> {
+        self.inner.proxy_capath(path)
+    }
+
+    /// Same as [`Easy2::proxy_sslcert`](struct.Easy2.html#method.proxy_sslcert)
+    pub fn proxy_sslcert(&mut self, sslcert: &str) -> Result<(), Error> {
+        self.inner.proxy_sslcert(sslcert)
+    }
+
+    /// Same as [`Easy2::proxy_sslkey`](struct.Easy2.html#method.proxy_sslkey)
+    pub fn proxy_sslkey(&mut self, sslkey: &str) -> Result<(), Error> {
+        self.inner.proxy_sslkey(sslkey)
+    }
+
     /// Same as [`Easy2::proxy_type`](struct.Easy2.html#method.proxy_type)
     pub fn proxy_type(&mut self, kind: ProxyType) -> Result<(), Error> {
         self.inner.proxy_type(kind)
@@ -586,6 +623,11 @@ impl Easy {
         self.inner.local_port_range(range)
     }
 
+    /// Same as [`Easy2::dns_servers`](struct.Easy2.html#method.dns_servers)
+    pub fn dns_servers(&mut self, servers: &str) -> Result<(), Error> {
+        self.inner.dns_servers(servers)
+    }
+
     /// Same as [`Easy2::dns_cache_timeout`](struct.Easy2.html#method.dns_cache_timeout)
     pub fn dns_cache_timeout(&mut self, dur: Duration) -> Result<(), Error> {
         self.inner.dns_cache_timeout(dur)
@@ -941,6 +983,15 @@ impl Easy {
         self.inner.ssl_version(version)
     }
 
+    /// Same as [`Easy2::ssl_min_max_version`](struct.Easy2.html#method.ssl_min_max_version)
+    pub fn ssl_min_max_version(
+        &mut self,
+        min_version: SslVersion,
+        max_version: SslVersion,
+    ) -> Result<(), Error> {
+        self.inner.ssl_min_max_version(min_version, max_version)
+    }
+
     /// Same as [`Easy2::ssl_verify_host`](struct.Easy2.html#method.ssl_verify_host)
     pub fn ssl_verify_host(&mut self, verify: bool) -> Result<(), Error> {
         self.inner.ssl_verify_host(verify)
@@ -1144,7 +1195,7 @@ impl Easy {
     }
 
     /// Same as [`Easy2::pipewait`](struct.Easy2.html#method.pipewait)
-    pub fn pipewait(&mut self, wait: bool) -> Result<(), Error>  {
+    pub fn pipewait(&mut self, wait: bool) -> Result<(), Error> {
         self.inner.pipewait(wait)
     }
 
@@ -1162,7 +1213,7 @@ impl Easy {
         // invoking `FnMut`closures behind a `&self` pointer. This flag acts as
         // our own `RefCell` borrow flag sorta.
         if self.inner.get_ref().running.get() {
-            return Err(Error::new(curl_sys::CURLE_FAILED_INIT))
+            return Err(Error::new(curl_sys::CURLE_FAILED_INIT));
         }
 
         self.inner.get_ref().running.set(true);
@@ -1266,13 +1317,14 @@ impl EasyData {
     /// Basically this is just intended to acquire a callback, invoke it, and
     /// then stop. Nothing else. Super unsafe.
     unsafe fn callback<'a, T, F>(&'a mut self, f: F) -> Option<&'a mut T>
-        where F: for<'b> Fn(&'b mut Callbacks<'static>) -> &'b mut Option<T>,
+    where
+        F: for<'b> Fn(&'b mut Callbacks<'static>) -> &'b mut Option<T>,
     {
         let ptr = self.borrowed.get();
         if !ptr.is_null() {
             let val = f(&mut *ptr);
             if val.is_some() {
-                return val.as_mut()
+                return val.as_mut();
             }
         }
         f(&mut self.owned).as_mut()
@@ -1325,11 +1377,7 @@ impl Handler for EasyData {
         }
     }
 
-    fn progress(&mut self,
-                dltotal: f64,
-                dlnow: f64,
-                ultotal: f64,
-                ulnow: f64) -> bool {
+    fn progress(&mut self, dltotal: f64, dlnow: f64, ultotal: f64, ulnow: f64) -> bool {
         unsafe {
             match self.callback(|s| &mut s.progress) {
                 Some(progress) => progress(dltotal, dlnow, ultotal, ulnow),
@@ -1358,7 +1406,8 @@ impl<'easy, 'data> Transfer<'easy, 'data> {
     /// Same as `Easy::write_function`, just takes a non `'static` lifetime
     /// corresponding to the lifetime of this transfer.
     pub fn write_function<F>(&mut self, f: F) -> Result<(), Error>
-        where F: FnMut(&[u8]) -> Result<usize, WriteError> + 'data
+    where
+        F: FnMut(&[u8]) -> Result<usize, WriteError> + 'data,
     {
         self.data.write = Some(Box::new(f));
         Ok(())
@@ -1367,7 +1416,8 @@ impl<'easy, 'data> Transfer<'easy, 'data> {
     /// Same as `Easy::read_function`, just takes a non `'static` lifetime
     /// corresponding to the lifetime of this transfer.
     pub fn read_function<F>(&mut self, f: F) -> Result<(), Error>
-        where F: FnMut(&mut [u8]) -> Result<usize, ReadError> + 'data
+    where
+        F: FnMut(&mut [u8]) -> Result<usize, ReadError> + 'data,
     {
         self.data.read = Some(Box::new(f));
         Ok(())
@@ -1376,7 +1426,8 @@ impl<'easy, 'data> Transfer<'easy, 'data> {
     /// Same as `Easy::seek_function`, just takes a non `'static` lifetime
     /// corresponding to the lifetime of this transfer.
     pub fn seek_function<F>(&mut self, f: F) -> Result<(), Error>
-        where F: FnMut(SeekFrom) -> SeekResult + 'data
+    where
+        F: FnMut(SeekFrom) -> SeekResult + 'data,
     {
         self.data.seek = Some(Box::new(f));
         Ok(())
@@ -1385,7 +1436,8 @@ impl<'easy, 'data> Transfer<'easy, 'data> {
     /// Same as `Easy::progress_function`, just takes a non `'static` lifetime
     /// corresponding to the lifetime of this transfer.
     pub fn progress_function<F>(&mut self, f: F) -> Result<(), Error>
-        where F: FnMut(f64, f64, f64, f64) -> bool + 'data
+    where
+        F: FnMut(f64, f64, f64, f64) -> bool + 'data,
     {
         self.data.progress = Some(Box::new(f));
         Ok(())
@@ -1394,7 +1446,8 @@ impl<'easy, 'data> Transfer<'easy, 'data> {
     /// Same as `Easy::ssl_ctx_function`, just takes a non `'static`
     /// lifetime corresponding to the lifetime of this transfer.
     pub fn ssl_ctx_function<F>(&mut self, f: F) -> Result<(), Error>
-        where F: FnMut(*mut c_void) -> Result<(), Error> + Send + 'data
+    where
+        F: FnMut(*mut c_void) -> Result<(), Error> + Send + 'data,
     {
         self.data.ssl_ctx = Some(Box::new(f));
         Ok(())
@@ -1403,7 +1456,8 @@ impl<'easy, 'data> Transfer<'easy, 'data> {
     /// Same as `Easy::debug_function`, just takes a non `'static` lifetime
     /// corresponding to the lifetime of this transfer.
     pub fn debug_function<F>(&mut self, f: F) -> Result<(), Error>
-        where F: FnMut(InfoType, &[u8]) + 'data
+    where
+        F: FnMut(InfoType, &[u8]) + 'data,
     {
         self.data.debug = Some(Box::new(f));
         Ok(())
@@ -1412,13 +1466,14 @@ impl<'easy, 'data> Transfer<'easy, 'data> {
     /// Same as `Easy::header_function`, just takes a non `'static` lifetime
     /// corresponding to the lifetime of this transfer.
     pub fn header_function<F>(&mut self, f: F) -> Result<(), Error>
-        where F: FnMut(&[u8]) -> bool + 'data
+    where
+        F: FnMut(&[u8]) -> bool + 'data,
     {
         self.data.header = Some(Box::new(f));
         Ok(())
     }
 
-    /// Same as `Easy::transfer`.
+    /// Same as `Easy::perform`.
     pub fn perform(&self) -> Result<(), Error> {
         let inner = self.easy.inner.get_ref();
 
@@ -1458,8 +1513,8 @@ impl<'easy, 'data> Transfer<'easy, 'data> {
 impl<'easy, 'data> fmt::Debug for Transfer<'easy, 'data> {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         f.debug_struct("Transfer")
-         .field("easy", &self.easy)
-         .finish()
+            .field("easy", &self.easy)
+            .finish()
     }
 }
 
