@@ -1,8 +1,8 @@
 use std::error;
 use std::ffi::{self, CStr};
 use std::fmt;
-use std::str;
 use std::io;
+use std::str;
 
 use curl_sys;
 
@@ -283,6 +283,16 @@ impl Error {
         self.code == curl_sys::CURLE_CHUNK_FAILED
     }
 
+    /// Returns whether this error corresponds to CURLE_HTTP2.
+    pub fn is_http2_error(&self) -> bool {
+        self.code == curl_sys::CURLE_HTTP2
+    }
+
+    /// Returns whether this error corresponds to CURLE_HTTP2_STREAM.
+    pub fn is_http2_stream_error(&self) -> bool {
+        self.code == curl_sys::CURLE_HTTP2_STREAM
+    }
+
     // /// Returns whether this error corresponds to CURLE_NO_CONNECTION_AVAILABLE.
     // pub fn is_no_connection_available(&self) -> bool {
     //     self.code == curl_sys::CURLE_NO_CONNECTION_AVAILABLE
@@ -293,6 +303,16 @@ impl Error {
         self.code
     }
 
+    /// Returns the general description of this error code, using curl's
+    /// builtin `strerror`-like functionality.
+    pub fn description(&self) -> &str {
+        unsafe {
+            let s = curl_sys::curl_easy_strerror(self.code);
+            assert!(!s.is_null());
+            str::from_utf8(CStr::from_ptr(s).to_bytes()).unwrap()
+        }
+    }
+
     /// Returns the extra description of this error, if any is available.
     pub fn extra_description(&self) -> Option<&str> {
         self.extra.as_ref().map(|s| &**s)
@@ -301,7 +321,7 @@ impl Error {
 
 impl fmt::Display for Error {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        let desc = error::Error::description(self);
+        let desc = self.description();
         match self.extra {
             Some(ref s) => write!(f, "[{}] {} ({})", self.code(), desc, s),
             None => write!(f, "[{}] {}", self.code(), desc),
@@ -312,22 +332,14 @@ impl fmt::Display for Error {
 impl fmt::Debug for Error {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         f.debug_struct("Error")
-         .field("description", &error::Error::description(self))
-         .field("code", &self.code)
-         .field("extra", &self.extra)
-         .finish()
+            .field("description", &self.description())
+            .field("code", &self.code)
+            .field("extra", &self.extra)
+            .finish()
     }
 }
 
-impl error::Error for Error {
-    fn description(&self) -> &str {
-        unsafe {
-            let s = curl_sys::curl_easy_strerror(self.code);
-            assert!(!s.is_null());
-            str::from_utf8(CStr::from_ptr(s).to_bytes()).unwrap()
-        }
-    }
-}
+impl error::Error for Error {}
 
 /// An error returned from "share" operations.
 ///
@@ -372,24 +384,9 @@ impl ShareError {
     pub fn code(&self) -> curl_sys::CURLSHcode {
         self.code
     }
-}
 
-impl fmt::Display for ShareError {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        error::Error::description(self).fmt(f)
-    }
-}
-
-impl fmt::Debug for ShareError {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(f, "ShareError {{ description: {:?}, code: {} }}",
-               error::Error::description(self),
-               self.code)
-    }
-}
-
-impl error::Error for ShareError {
-    fn description(&self) -> &str {
+    /// Returns curl's human-readable version of this error.
+    pub fn description(&self) -> &str {
         unsafe {
             let s = curl_sys::curl_share_strerror(self.code);
             assert!(!s.is_null());
@@ -398,6 +395,25 @@ impl error::Error for ShareError {
     }
 }
 
+impl fmt::Display for ShareError {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        self.description().fmt(f)
+    }
+}
+
+impl fmt::Debug for ShareError {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        write!(
+            f,
+            "ShareError {{ description: {:?}, code: {} }}",
+            self.description(),
+            self.code
+        )
+    }
+}
+
+impl error::Error for ShareError {}
+
 /// An error from "multi" operations.
 ///
 /// THis structure wraps a `CURLMcode`.
@@ -456,24 +472,9 @@ impl MultiError {
     pub fn code(&self) -> curl_sys::CURLMcode {
         self.code
     }
-}
 
-impl fmt::Display for MultiError {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        error::Error::description(self).fmt(f)
-    }
-}
-
-impl fmt::Debug for MultiError {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(f, "MultiError {{ description: {:?}, code: {} }}",
-               error::Error::description(self),
-               self.code)
-    }
-}
-
-impl error::Error for MultiError {
-    fn description(&self) -> &str {
+    /// Returns curl's human-readable description of this error.
+    pub fn description(&self) -> &str {
         unsafe {
             let s = curl_sys::curl_multi_strerror(self.code);
             assert!(!s.is_null());
@@ -482,6 +483,22 @@ impl error::Error for MultiError {
     }
 }
 
+impl fmt::Display for MultiError {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        self.description().fmt(f)
+    }
+}
+
+impl fmt::Debug for MultiError {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        f.debug_struct("MultiError")
+            .field("description", &self.description())
+            .field("code", &self.code)
+            .finish()
+    }
+}
+
+impl error::Error for MultiError {}
 
 /// An error from "form add" operations.
 ///
@@ -536,24 +553,9 @@ impl FormError {
     pub fn code(&self) -> curl_sys::CURLFORMcode {
         self.code
     }
-}
 
-impl fmt::Display for FormError {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        error::Error::description(self).fmt(f)
-    }
-}
-
-impl fmt::Debug for FormError {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(f, "FormError {{ description: {:?}, code: {} }}",
-               error::Error::description(self),
-               self.code)
-    }
-}
-
-impl error::Error for FormError {
-    fn description(&self) -> &str {
+    /// Returns a human-readable description of this error code.
+    pub fn description(&self) -> &str {
         match self.code {
             curl_sys::CURL_FORMADD_MEMORY => "allocation failure",
             curl_sys::CURL_FORMADD_OPTION_TWICE => "one option passed twice",
@@ -569,9 +571,29 @@ impl error::Error for FormError {
     }
 }
 
+impl fmt::Display for FormError {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        self.description().fmt(f)
+    }
+}
+
+impl fmt::Debug for FormError {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        f.debug_struct("FormError")
+            .field("description", &self.description())
+            .field("code", &self.code)
+            .finish()
+    }
+}
+
+impl error::Error for FormError {}
+
 impl From<ffi::NulError> for Error {
     fn from(_: ffi::NulError) -> Error {
-        Error { code: curl_sys::CURLE_CONV_FAILED, extra: None }
+        Error {
+            code: curl_sys::CURLE_CONV_FAILED,
+            extra: None,
+        }
     }
 }
 
